//
//  COGlyphRecord.h
//  GSFontTools
//
//  Created by Georg Seifert on 04.08.11.
//  Copyright 2011 schriftgestaltung.de. All rights reserved.
//

#import <Cocoa/Cocoa.h>
@class OTValueRecord;

NSMutableArray * glyphNamesToGlyphRecords(NSArray * glyphList) ;
NSMutableArray * glyphRecordsToGlyphNames(NSArray* glyphRecords) ;

@interface COGlyphRecord : NSObject {
	id _glyph;
	NSString * _glyphName;
	NSInteger _xPlacement;
	NSInteger _yPlacement;
	NSInteger _xAdvance;
	NSInteger _yAdvance;
	NSInteger _advanceWidth;
	NSInteger _advanceHeight;
	NSMutableArray * _alternates;
	NSString * _alternatesReference;
	NSArray * _ligatureComponents;
	NSArray * _ligatureComponentsReference;
	NSMutableArray * _substitutionHistory;
	
	NSDictionary * _attributes;
}
@property (retain, nonatomic) NSString * glyphName;
@property (retain, nonatomic) NSArray * ligatureComponents;
@property (retain, nonatomic) NSMutableArray * alternates;
@property (retain, nonatomic) NSString * alternatesReference;
@property (nonatomic, retain) NSDictionary *attributes;
@property (nonatomic, retain) NSMutableArray *substitutionHistory;

/**
 Similar to `substitutionHistory` but for characters which expand to multiple glyphs.
 
 This should be set on glyphs records before they are removed from the collection that is being processed. Before processing, clients can save a shallow copy of this collection which remains unprocessed, so changes to the records can still be reflected even though we have not changed the API to keep the passed-in glyph record collections immutable. In the end, the glyph counts in the two collections can be compared record for record.
 
 @note This is a non-standard extension.
 */
@property (nonatomic, readonly) NSArray *substitutionFuture;

@property (nonatomic, assign) NSInteger xPlacement;
@property (nonatomic, assign) NSInteger yPlacement;
@property (nonatomic, assign) NSInteger xAdvance;
@property (nonatomic, assign) NSInteger yAdvance;
@property (nonatomic, assign) NSInteger advanceWidth;
@property (nonatomic, assign) NSInteger advanceHeight;

- (id) initWithGlyphName:(NSString*) GlyphName ;
- (void) addValues:(OTValueRecord*) aValueRecord ;

/**
 Appends the given glyph names to the receiver’s substitution history.

 @param GlyphNames An array of glyph names (as NSString objects) to append.
 */
- (void) saveState:(NSArray*) GlyphNames ;

/**
 Appends the given glyph names to the receiver’s substitution future.
 
 @note This is a non-standard extension.

 @param glyphNames An array of glyph names (as NSString objects) to append.
 */
- (void)saveFutureState:(NSArray*)glyphNames;

- (NSString*) side1GlyphNameWithUnicodeValue:(NSDictionary*) reversedCMAP;
- (NSString*) side2GlyphNameWithUnicodeValue:(NSDictionary*) reversedCMAP;
@end
